@@ -1,15 +1,15 @@
 package XFST::Lookup;
 
-# $Header: /var/cvsroot/XFST-Lookup/lib/XFST/Lookup.pm,v 1.29 2005/07/24 15:53:17 alex Exp $
+# $Header: /var/cvsroot/XFST-Lookup/lib/XFST/Lookup.pm,v 1.11 2005/06/10 13:39:37 alex Exp $
 
-$VERSION="0.11";
+$VERSION="0.4";
 
 use strict;
 use Carp;
 
 =head1 NAME
 
-XFST::Lookup - Perl extension to access *XEROX*'s lookup-utility
+XFST::Lookup - PERL extension to access *XEROX*'s lookup-utility
 
 =head1 SYNOPSIS
     
@@ -33,7 +33,7 @@ XFST::Lookup - Perl extension to access *XEROX*'s lookup-utility
 =head1 DESCRIPTION
 
 XFST::Lookup() provides an easy way to access the functionality of the
-*XEROX* lookup-utility from within Perl.
+*XEROX* lookup-utility from within PERL.
 I<lookup> itself "applies a pre-compiled transducer or transducers to
 look up words".
 
@@ -62,40 +62,23 @@ sub new {
 sub _check4bin {
     my $self = shift;
     my $binary = shift;
-    my $path2bin = "";
-    my $found = 0;
+    my $path2bin;
     
     print "Looking for '$binary'... " if $self->{debug};
 
     # try to locate xfst's lookup (or any other binary) (in $ENV{PATH})
-    if ( $^O =~ /linux|bsd|darwin|cygwin/ ) {
+    # TODO: what about win32? steffi?
+    if ( $^O =~ /linux|bsd|darwin/ ) {
         # got a user-defined path? add to $ENV{PATH}
         $ENV{PATH} .= ":$self->{path}" if $self->{path};
         
         # give 'which' a try...
         chomp( $path2bin = `which $binary 2>/dev/null` );
-    
-        ( $? == 0 ) ? $found = 0 : $found = 1;
+        
+        croak "Cannot locate '$binary' in \$PATH ('$ENV{PATH}').\n" .
+            "\nYou can add a user-defined path via the 'path'-parameter.\n"
+            if ( $? != 0 );
     }
-    elsif ( $^O eq "MSWin32" ) {
-        my $ms_binary="$binary\.exe";
-        if( system("$ms_binary -v") == 0) {
-            print "MSWin32: $ms_binary found in standard-path...\n"
-                if $self->{debug};
-            $found=1;
-            $path2bin=$ms_binary;
-        }
-        elsif( $self->{path}  && system("$self->{path}\\$ms_binary -v") == 0 ) {
-            print "MSWin32: $ms_binary found in user-defined path: " .
-                "'$self->{path}'\n" if $self->{debug};
-            $found = 1;
-            $path2bin="$self->{path}\\$ms_binary";
-        }
-    }
-    
-    croak "Cannot locate '$binary' in \$PATH ('$ENV{PATH}').\n" .
-        "\nYou can add a user-defined path via the 'path'-parameter.\n"
-        unless( $found );
     
     print "$path2bin\n" if $self->{debug};
 
@@ -112,14 +95,8 @@ sub _check_lookup {
 
     my $version_str = `$bin -v 2>&1`;
     $version_str =~ /lookup ([\d\.]+)/;
-    
-    if (defined $1) {
-        $self->{lookup_version} = $1;
-    }
-    else {
+    (defined $1) ? $self->{lookup_version} = $1 :
         carp "Cannot determine lookup-version!\n";
-        $self->{lookup_version} = ">unknown<";
-    }
 
     # do version specific stuff here if necessary...
 
@@ -133,7 +110,7 @@ sub _check_lookup {
     
 You may provide parameters via an anonymous hashref like this:
 
-    my $lookup = new XFST::Lookup( {
+    my $test = new XFST::Lookup( {
             debug => 1 ,
             flags => 'some flags',
             path => '/home/alex/bin/:/home/roni/bin/',
@@ -162,12 +139,8 @@ you won't have to set it every time you use lookup()
 
 =item I<path> [path]
 
-I<Linux/*BSD/MacOS>:
-
-=over 2
-
-Specify a single addition or a list of additions to $PATH in order to help
-XFST::Lookup to locate the lookup-binary. (separator: ':')
+Specify an extra-$PATH or a list of $PATHes in order to help XFST::Lookup
+to locate the lookup-binary. (separator: ':')
 
 Per default XFST::Lookup is looking for the binary in $ENV{PATH} on
 Linux,*BSD and Mac OS calling the which(1)-utility that is shipped with
@@ -175,30 +148,17 @@ every *NIX-distribution.
 
 =back
 
-I<Win32>:
-
-=over 2
-
-On MSWin32 XFST::Lookup tries to locate the binary in the defaut path or
-searches a user-defined path afterwards (i.e. supplied via
-"path=>d:\alex\xerox-windows\")
-
-Manually setting the path on this OS is recommended...
-
-=back
-
 =head1 METHODS
 
 =head2 B<set_flags()> [@array|$scalar]
 
 =over 2
 
-set_flags() is used to set the flags that will be used by *XEROX*'s
+set_flags() is used to set the flags that'll be used by *XEROX*'s
 lookup-utility. See [3] for a brief overview.
 
-Please note: even if you set them manually, the 'LxL' and 'TxT'-flags are
-stripped, in order to do what you mean. XFST::Lookup needs the default
-separators in order to correctly process the output of lookup.
+Please note: the 'LxL' and 'TxT'-flags are stripped, even if you set them
+manually in order to do what you mean (I at least hope so) ;)
 
 =back
 
@@ -218,63 +178,25 @@ sub set_flags {
     $self->{lookup_flags} = $flags;
 }
 
-=back
-
 =head2 B<lookup()> [{anonymous hashref}]
 
 =over 2
 
 ... is the most important method of XFST::Lookup (like the name may already
-suggest). Every time lookup() is invoked from the inside of a Perl-program,
+suggest). Every time lookup() is invoked from the inside of a PERL-program,
 *XEROX*'s lookup-utility is called with the given options and all results
 are returned as a I<hash of arrays>. This datastructure has been chosen
-because there may be ambiguous wordforms so a simple hash won't fulfill
+because there may be ambiguous wordforms so a simple hash won't fullfil
 the needs.
 
-Here's an example of the mentioned datastructure (Data::Dumper-format):
-
-    $VAR1 = 'eine';
-    $VAR2 = [
-              'ein+ART+NOM+SG',
-              'ein+ART+AKK+SG',
-              'ein+PIS+AKK+SG',
-              'ein+PIS+NOM+SG',
-              'ein+PIAT+AKK+SG',
-              'ein+PIAT+NOM+SG',
-              'eine+PIDAT'
-            ];
-    $VAR3 = 'Haus';
-    $VAR4 = [
-              'Haus+NN+NOM+SG',
-              'Haus+NN+DAT+SG',
-              'Haus+NN+AKK+SG'
-            ];
-    $VAR5 = 'das';
-    $VAR6 = [
-              'das+ART+NOM+SG',
-              'das+ART+AKK+SG',
-              'das+KOUS',
-              'der+PDAT+NOM+SG',
-              'der+PDAT+AKK+SG',
-              'der+PDS+NOM+SG',
-              'der+PDS+AKK+SG'
-            ];
-    ...
-
-While the hash's key contains a I<wordform>, the value provides an @array
-of all analyzed I<readings> of this wordform.
-
-The following parameters are available to alter XFST::Lookup's runtime
-behaviour:
-
 =over 2
 
 =item I<fst> [path]
 
-set the path to the file that contains the finite state transducer to use
+set the patch to the file that contains the finite state transducer to use
 ('save stack'-format).
 
-I<fst> is optional at this point iff it was already passed to the constructor,
+B<fst> is optional at this point iff it was already passed to the constructor,
 it is forbidden if you use the I<strategy>-parameter at the same time.
 
 Please note: Setting B<fst> again at this point will overwrite the setting for
@@ -282,7 +204,7 @@ the current run _only_. see CONSTRUCTOR
 
 =item I<strategy> [path]
 
-set parameter to the path to the strategy-file if you want to use a
+set parameter to the path to the stratgy-file if you want to use a
 lookup-strategy rather than a single finite state transducer.
 
 Conflicts with the I<fst>-parameter.
@@ -305,8 +227,6 @@ sub lookup {
     my $opt = shift || croak "No options given!\n";
     
     my %results;
-
-    my %seen;
     
     croak "Options not supplied via hashref!\n" unless ( ref($opt) eq 'HASH' );
     
@@ -324,129 +244,83 @@ sub lookup {
     my $lookup_file;
     if ( $opt->{fst} ) {
         $lookup_file= $opt->{fst};
-        print STDERR "Using a single FST ('$lookup_file').\n"
-            if $self->{debug};
-        croak "File '$lookup_file' doesn't exist!\n" unless -e $lookup_file;
+        print STDERR "Using a single FST ('$lookup_file').\n" if $self->{debug};
     }
     else {
-        croak "File '$lookup_file' doesn't exist!\n"
-            unless -e $opt->{strategy};
         $lookup_file= "-f " . $opt->{strategy};
-        print STDERR "Using a lookup-strategy ('$opt->{strategy}').\n"
+        print STDERR "Using a lookup-strategy ('$lookup_file').\n"
             if $self->{debug};
     }
-
+   
     print "Using FST '$lookup_file'...\n", scalar(@{$opt->{words}}),
         " words to look up.\n" if $self->{debug};
 
-    # Any extra flags that shall be passed to *XEROX*'s lookup?
+    # any extra flags that shall be passed to *XEROX*'s lookup?
     my $flags;
-    unless ( $self->{lookup_flags} ) { $flags = '' }
-    else { $flags="-flags" . ' ' . $self->{lookup_flags} }
+    ( $self->{lookup_flags} ) ? $flags = '' :
+        $flags="-flags $self->{lookup_flags}";
    
-    # Strip lookup's 'LL' and 'TT'-flags (they could break _everything_
+    # Strip lookup's 'LL' and 'TT'-flags (they could break _anything_
     # later on...)
     if ( $flags =~ /T[^T]*T|L[^L]*L/g ) {
         $flags =~ s/(T[^T]*T)|(L[^L]*L)//g;
         print STDERR "Stripping flags... (new: '$flags')\n" if $self->{debug};
     }
 
-    # Where shall we direct lookup's STDERR output to?
-    # We don't want to parse that one anyway... (infos, stats)
-    # NOTE: Doesn't work on MS Windows...
+    # where shall we direct lookup's STDERR output to?
+    # we don't want to parse that one anyway... (infos, stats)
+    # TODO: what about win32? :(
     my $dest_stream;
     ( $self->{debug} == 1 ) ? $dest_stream = '&2' :  
-        ( ( $^O =~ /linux|bsd|darwin/i ) ?
+        ( ( $^O =~ /linux|bsd|mac/i ) ?
             $dest_stream = '&2' : $dest_stream = '/dev/null' );
     
-    # Using fork() and stuff in order to get rid of File::Temp...
-    # ...iff not running on MS Windows...
-    
-    my ( $pid, @output );
+    # using fork() and stuff in order to get rid of File::Temp...
 
-    if ( $^O eq "MSWin32" ) {
-        # This is a hack - I am aware of that :(
-        # I don't know how to do it any better on Win32, because I am not
-        # familiar with that OS at all.
+    my ( $pid, @output ); 
+    # coming up next: fork()-magic ;-)
+    if( $pid = open( CHILD, "-|")) { # -> parent
+        print STDERR "Child forked. PID is $pid.\n" if $self->{debug};
 
-        # Here's what went wrong/didn't work/caught fire:
-        #  o Fcntl (flock) doesn't work at all (see perlport-function)
-        #  o wait/waitpid doesn't work as expected (doesn't seem to threat
-        #    a pipe as a child-process - but it should (according to
-        #    perlport-function again), because perl is able to emulate fork()
-        #    although win32 generally lacks support for this syscall.
-        #  o IPC::Open2 didn't work well - in fact it refused to let me read
-        #    any data back from the pipe
-        #    
-        #    ...so File::Temp and a pseudo-cat are the last resort :(       
+        chomp( @output = <CHILD> );
+
+        close CHILD;
+    }
+    else { # -> child
+        croak "fork() failed: '$!'\n" unless defined $pid;
+        print STDERR "Child executes $self->{lookup_binary}." .
+            " Output goes to parent.\n" if $self->{debug};
+
+        # open the pipe to lookup...
+        open PIPE,
+            "| $self->{lookup_binary} $lookup_file $flags 2>$dest_stream"
+                or croak "Cannot fork(): $self->{lookup_binary}: '$!'\n";
+        # ...and feed in the wordforms.
+        print PIPE join("\n",@{$opt->{words}}), "\n";
         
-        print "Using File::Temp on Win32 :(\n" if $self->{debug};
-
-        use File::Temp;
-        
-        my $tmp = new File::Temp( UNLINK => 1, SUFFIX => '.txt' );
-
-        print "Tempfile is '$tmp'\n" if $self->{debug};
-
-        # write all wordforms to the tempfile...
-        print $tmp join("\n", @{$opt->{words}}), "\n";
-        
-        # ... use Perl as kind of cat-command to pipe the wordforms to
-        # lookup. This looks terribly ugly, but the Win32 echo-command
-        # doesn't even seem to handle escape-sequences, which are required
-        # in order to get some useful results out of lookup.
-        open WINPIPE,
-            "perl -e \"open FH, '$tmp' or die; print foreach <FH>\"" . 
-            " | $self->{lookup_binary} $lookup_file $flags |" or
-            die "Failed to open pipes: '$!'\n";
-
-        @output = <WINPIPE>;
-    }   
-    else {  
-        # running on *NIX -- using fork()
-        if( $pid = open( CHILD, "-|") ) { # -> parent
-            print STDERR "Child forked. PID is $pid.\n" if $self->{debug};
-
-            chomp( @output = <CHILD> );
-
-            close CHILD;
-        }
-        else { # -> child
-            croak "fork() failed: '$!'\n" unless defined $pid;
-            print STDERR "Child executes $self->{lookup_binary}." .
-                " Output goes to parent.\n" if $self->{debug};
-
-            # open the pipe to lookup...
-            open PIPE,
-                "| $self->{lookup_binary} $lookup_file $flags 2>$dest_stream"
-                    or croak "Cannot fork(): $self->{lookup_binary}: '$!'\n";
-            # ...and feed in the wordforms.
-            print PIPE join("\n",@{$opt->{words}}), "\n";
-        
-            # terminate child (continuing under parent-pid)
-            exit;
-        }
+        # terminate child (continuing under parent-pid)
+        exit;
     }
 
+    # the follwing class describes all chars that are valid for the
+    # transducer's output.
+    my $cclass = '\w\d\.-_\+!\"\'\\\//';
+    
     # extract information from lookup's output
     foreach my $line ( @output ) {
-        # cygwin's lookup-output contains "\r", so we can't just chomp()
-        $line =~ s/[\n\r]//g;
+        chomp $line;
         next unless $line;
 
         # match the three fields ( 1x input, 2x parts analysis(base+tags) )
-        $line =~ /^([^\s]+)\s+([^\s]+)\s+([^\s]+)$/;
-        if ( $1 && $2 && $3 ) {
-            unless( $seen{$1}->{"$2$3"} ) {
-                push @{$results{$1}}, "$2$3";
-                $seen{$1}->{"$2$3"} = 1;
-            }
+        $line =~ /^([$cclass]+)\s+([$cclass]+)\s+([$cclass]+)$/;
+        if( $1 && $2 && $3 ) {
+            push @{$results{$1}}, "$2$3";
         }
         else {
             carp "Error parsing line: '$line'\n";
         }
     }
-
+    
     return %results;
 }
 
